# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

from django.conf.urls import url
from django.contrib import admin
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _

from location.inlines import CountryTranslationsTabularInline, StateTranslationTabularInline, \
    CityTranslationTabularInline
from .forms import LocationForm, CountryForm, CityForm, StateForm
from .models import *


class CityAdmin(admin.ModelAdmin):
    form = CityForm
    inlines = (CityTranslationTabularInline,)


admin.site.register(City, CityAdmin)


class StateAdmin(admin.ModelAdmin):
    form = StateForm
    inlines = (StateTranslationTabularInline,)


admin.site.register(State, StateAdmin)


class CountryAdmin(admin.ModelAdmin):
    form = CountryForm
    inlines = (CountryTranslationsTabularInline,)


admin.site.register(Country, CountryAdmin)


class LocationAdmin(admin.ModelAdmin):
    change_form_template = 'admin/location/change_form_location.html'
    form = LocationForm
    model = Location
    prepopulated_fields = {"slug": ("address",)}
    fieldsets = (
        (
            _('Location'), {
                'fields': (
                    'map',
                    'field1',
                    'country',
                    'states',
                    'city',
                    'neighborhood',
                    'postal_code',
                    'address',
                    'street_number',
                    'complement',
                    'latitude',
                    'longitude',
                )
            }
        ),
    )

    def __init__(self, model, admin_site):
        self.form.admin_site = admin_site
        super(LocationAdmin, self).__init__(model, admin_site)

    def get_urls(self):
        urls = super(LocationAdmin, self).get_urls()
        my_urls = [
            url(r'^cities/', self.admin_site.admin_view(self.cities), name='city'),
            url(r'^states/', self.admin_site.admin_view(self.states), name='state'),
        ]
        return my_urls + urls

    def cities(self, request):
        cities = [{'id': city.pk, 'name': city.name} for city in
                  City.objects.filter(state=request.GET.get('state')).all()]
        return HttpResponse(json.dumps(cities),
                            content_type='application/json')

    def states(self, request):
        states = [{'id': state.pk, 'name': state.name} for state in
                  State.objects.filter(country=request.GET.get('country')).all()]
        return HttpResponse(json.dumps(states), content_type='application/json')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from rest_framework import serializers
from .models import Country, State, City, Location


class CountrySerializer(serializers.ModelSerializer):
    class Meta:
        model = Country
        fields = (
            'id',
            'created_at',
            'updated_at',
            'active',
            'slug',
            'code2',
            'code3',
            'flag',
            'name',
        )


class StateSerializer(serializers.ModelSerializer):
    country = CountrySerializer(many=False)

    class Meta:
        model = State
        fields = (
            'id',
            'country',
            'created_at',
            'updated_at',
            'active',
            'code',
            'name',
        )


class CitySerializer(serializers.ModelSerializer):
    state = StateSerializer(many=False)

    class Meta:
        model = City
        fields = (
            'id',
            'state',
            'created_at',
            'updated_at',
            'active',
            'name',
        )


class LocationSerializer(serializers.ModelSerializer):
    city = CitySerializer(many=False)

    class Meta:
        model = Location
        fields = '__all__'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django_filters import FilterSet

from ..models import Country, State, City


class CountryFilterSet(FilterSet):
    class Meta:
        model = Country
        fields = (
            'code2',
            'code3'
        )


class StateFilterSet(FilterSet):

    class Meta:
        model = State
        fields = (
            'code',
            'country',
            'country__code2',
            'country__code3'
        )


class CityFilterSet(FilterSet):

    class Meta:
        model = City
        fields = (
            'state',
            'state__code',
            'state__country',
            'state__country__code2',
            'state__country__code3'
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-04-09 13:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0004_auto_20180409_1319'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
        migrations.AlterField(
            model_name='citytranslation',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
        migrations.AlterField(
            model_name='country',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
        migrations.AlterField(
            model_name='countrytranslations',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
        migrations.AlterField(
            model_name='state',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
        migrations.AlterField(
            model_name='statetranslation',
            name='geocode',
            field=models.IntegerField(blank=True, default=0, editable=False, null=True, unique=True, verbose_name='Geocode'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _, get_language


class Control(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, blank=False)
    updated_at = models.DateTimeField(auto_now=True, blank=False)
    active = models.BooleanField(default=True)

    class Meta:
        abstract = True


class Country(Control):
    slug = models.CharField(max_length=255, blank=False, unique=True, editable=False)
    code2 = models.CharField(_('Código - alpha2'), max_length=2, blank=False, unique=True, default='')
    code3 = models.CharField(_('Código - alpha3'), max_length=3, blank=False, unique=True, default='')
    flag = models.FileField(_('Bandeira'), blank=True, null=True)
    geocode = models.IntegerField(_('Geocode'), null=True, blank=True, unique=True, default=0, editable=False)

    class Meta:
        verbose_name = _('País')
        verbose_name_plural = _('Paises')

    def __unicode__(self):
        return unicode(self.name)

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        self.slug = slugify(self.code2)
        return super(Country, self).save(force_insert=force_insert, force_update=force_update, using=using,
                                         update_fields=update_fields)

    @property
    def name(self):
        try:
            return str(CountryTranslations.objects.get(country__pk=self.pk, lang=get_language()))
        except Exception as e:
            return self.code2


class CountryTranslations(Control):
    lang = models.CharField(max_length=10, choices=settings.LANGUAGES)
    country = models.ForeignKey(Country, blank=False, related_name='country_translation', verbose_name=_('Pais'))
    name = models.CharField(max_length=50, blank=False)

    class Meta:
        verbose_name = _('Nome do País')
        verbose_name_plural = _('Nome dos Paises')
        unique_together = (("lang", "country"),)

    def __unicode__(self):
        return unicode(self.name)


class State(Control):
    country = models.ForeignKey(Country, blank=False, verbose_name=_('Pais'))
    code = models.CharField(max_length=50)
    geocode = models.IntegerField(_('Geocode'), null=True, blank=True, unique=True, default=0, editable=False)

    class Meta:
        verbose_name = _('Estado')
        verbose_name_plural = _('Estados')

    def __unicode__(self):
        return unicode(self.name)

    @property
    def name(self):
        try:
            return str(StateTranslation.objects.get(state__pk=self.pk, lang=get_language()))
        except Exception as e:
            return unicode(str(self.geocode) + ' - ' + self.country.code2.upper())


class StateTranslation(Control):
    lang = models.CharField(max_length=10, choices=settings.LANGUAGES)
    state = models.ForeignKey(State, blank=False, related_name='state_translation', verbose_name=_('Região/Estado'))
    name = models.CharField(max_length=50)

    class Meta:
        verbose_name = _('Nome Região/Estado')
        verbose_name_plural = _('Nome Regiões/Estados')
        unique_together = (("lang", "state"),)

    def __unicode__(self):
        return unicode(self.name)


class City(Control):
    state = models.ForeignKey(State, blank=False)
    geocode = models.IntegerField(_('Geocode'), null=True, blank=True, default=0, editable=False)
    latitude = models.DecimalField(max_digits=8, decimal_places=5, null=True, blank=True)
    longitude = models.DecimalField(max_digits=8, decimal_places=5, null=True, blank=True)

    class Meta:
        verbose_name = _('Cidade')
        verbose_name_plural = _('Cidades')

    def __unicode__(self):
        return unicode(self.name)

    @property
    def name(self):
        try:
            city_translate = CityTranslation.objects.get(city__pk=self.pk, lang=get_language())
            return unicode(city_translate)
        except Exception:
            return unicode(u'cidade - %s, %s - %s' % (self.pk, self.state.code, self.state.country.code2))


class CityTranslation(Control):
    lang = models.CharField(max_length=10, choices=settings.LANGUAGES)
    city = models.ForeignKey(City, blank=False, related_name='city_translation', verbose_name=_('Cidade'))
    name = models.CharField(max_length=50)

    class Meta:
        verbose_name = _('Nome de Cidade')
        verbose_name_plural = _('Nomes de Cidades')
        unique_together = (("lang", "city"),)

    def __unicode__(self):
        return unicode(self.name)

    def __str__(self):
        return str(self.name)


class Location(Control):
    city = models.ForeignKey(City, verbose_name=_('Cidade'))
    neighborhood = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Bairro'))
    postal_code = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Código Postal'))
    address = models.CharField(max_length=255, blank=False, verbose_name=_('Endereço'))
    street_number = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Número'))
    complement = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Complemento'))
    latitude = models.CharField(max_length=255, verbose_name=_('Latitude'))
    longitude = models.CharField(max_length=255, verbose_name=_('Longitude'))
    slug = models.CharField(max_length=255, blank=False, unique=False)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.address



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-04-09 13:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0005_auto_20180409_1322'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='citytranslation',
            unique_together=set([('lang', 'city')]),
        ),
        migrations.AlterUniqueTogether(
            name='countrytranslations',
            unique_together=set([('lang', 'country')]),
        ),
        migrations.AlterUniqueTogether(
            name='statetranslation',
            unique_together=set([('lang', 'state')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets, filters

from .api.filters import CityFilterSet, CountryFilterSet, StateFilterSet
from .models import City, State, Country, Location
from .serializers import CitySerializer, StateSerializer, CountrySerializer, LocationSerializer


class CityViewSet(viewsets.ModelViewSet):
    queryset = City.objects.all()
    serializer_class = CitySerializer
    filter_class = CityFilterSet
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter,)
    ordering_fields = '__all__'
    http_method_names = ['get', ]


class StateViewSet(viewsets.ModelViewSet):
    queryset = State.objects.all()
    serializer_class = StateSerializer
    filter_class = StateFilterSet
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter,)
    ordering_fields = '__all__'
    http_method_names = ['get', ]


class CountryViewSet(viewsets.ModelViewSet):
    queryset = Country.objects.all()
    serializer_class = CountrySerializer
    filter_class = CountryFilterSet
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter,)
    ordering_fields = '__all__'
    http_method_names = ['get', ]


class LocationViewSet(viewsets.ModelViewSet):
    model = Location
    serializer_class = LocationSerializer
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter,)
    ordering_fields = '__all__'
    http_method_names = ['get', ]




# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-04-09 14:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0008_auto_20180409_1328'),
    ]

    operations = [
        migrations.AddField(
            model_name='city',
            name='latitude',
            field=models.DecimalField(blank=True, decimal_places=5, max_digits=8, null=True),
        ),
        migrations.AddField(
            model_name='city',
            name='longitude',
            field=models.DecimalField(blank=True, decimal_places=5, max_digits=8, null=True),
        ),
    ]



from rest_framework import routers

from ..views import CityViewSet, StateViewSet, CountryViewSet

router = routers.DefaultRouter()
router.register(r'cities', CityViewSet)
router.register(r'states', StateViewSet)
router.register(r'countries', CountryViewSet)